/*
 * CreateObjectFromXML.java
 *
 * Created on December 12, 2004, 10:26 PM
 */

package university;
import java.util.HashMap;
/**
 * Creates the Objects from the values that were parsed from the XML 
 * document.
 * @author  Nathan Balon
 * University of Michigan Dearborn
 * CIS 550 Object Oriented Programming
 */
public class BuildPersonFromXML {
    
    /** Creates a new instance of BuildPersonFromXML */
    public BuildPersonFromXML() {
    }
    /**
     * Creates an Address object from the values read in from the xml
     * document.
     * @param map A map containing all of the values need to create
     * a new Address object. 
     * @return Address The new Address that was created.
     */
    public Address createAddress(HashMap <String, String> map){
        String street = map.get("street");
        String city = map.get("city");
        String state = map.get("state");
        String zipcode = map.get("zipcode");
        return new Address(street, city, state, zipcode);
    }    
    /**
     * Creates a new Person from the values read in from the xml document.
     * @param personMap A map contain all of the values read in from the
     * xml document.  The map contains the 
     * @return Person The Person that was created.
     */
    public Person createPerson(HashMap <String, String> personMap){
        String name = personMap.get("name");
        String gender = personMap.get("gender");
        Sex sex = this.getGender(gender);
        String unitName = personMap.get("unit");
        Unit unit = getUnit(unitName);
        Address address = this.createAddress(personMap);
        return new Person(name, sex, address, unit); 
    }
    
    /**
     * Create a new Undergraduate student from the values read in from 
     * the xml document.
     * @param undergraduateMap A HashMap containing all of the values to
     * construct a new Undergraduate. 
     * @return Undergraduate The Undergraduate student that was created.
     */
    public Undergraduate createUndergraduate(HashMap <String, String> undergraduateMap){
        String name = undergraduateMap.get("name");
        String gender = undergraduateMap.get("gender");
        Sex sex = this.getGender(gender);
        String unitName = undergraduateMap.get("unit");
        Unit unit = getUnit(unitName);
        Address address = this.createAddress(undergraduateMap);
        String level = undergraduateMap.get("status");
        String gpa = undergraduateMap.get("gpa");
        UndergraduateLevel undergraduateLevel = this.getUndergraduateLevel(level);
        return new Undergraduate(name, sex, address, unit, undergraduateLevel, gpa);        
    }

    /**
     * Create a new Graduate student from the values read in from the
     * xml document.
     * @param graduateMap A HashMap containing all of the values needed
     * to construct a Graduate student. 
     * @return Graduate The new Graduate student that was created.
     */
    public Graduate createGraduate(HashMap <String, String> graduateMap){
        String name = graduateMap.get("name");
        String gender = graduateMap.get("gender");
        Sex sex = this.getGender(gender);
        String unitName = graduateMap.get("unit");
        Unit unit = getUnit(unitName);
        Address address = this.createAddress(graduateMap);
        String level = graduateMap.get("status");
        GraduateLevel graduateLevel = this.getGraduateLevel(level);
        String research = graduateMap.get("researchTopic");
        return new Graduate(name, sex, address, unit, graduateLevel, research);         
    }  

    /**
     * Returns a Unit object from the value read in from the xml document.
     * @param unitName The name of the Unit object to get a reference to. 
     * @return Unit The unit belonged to.
     */
    public Unit getUnit(String unitName){
        if(unitName.equals("cis")){
            return UnitsFactory.getCIS();
        } else if (unitName.equals("education")){
            return UnitsFactory.getEducation();
        } else if (unitName.equals("hr")){
            return UnitsFactory.getHR();
        } else if (unitName.equals("addmissions")){
            return UnitsFactory.getAddmissions();
        } else  if (unitName.equals("library")){
            return UnitsFactory.getLibrary();
        }
        return null;
    }
    
    /**
     * getGender return a Sex object for the String supplied
     * @param gender A String contain the value "male" or "female"
     * @return Sex the enum value of the person gender.
     */
    public Sex getGender(String gender){
        Sex sex = null;
        if(gender.equals("male")){
            sex = Sex.MALE;
        } else if(gender.equals("female")){
            sex = Sex.FEMALE;
        } else {
            System.out.println("Invalid input for gender");
        }
        return sex;
    }
    
     /**
      * getGraduateLevel returns a GraduateLevel object for the String.
      * @param level A String for the graduate level can contain the 
      * value "masters" or "phd"
      * @return GraduateLevel The Graduate student grade level.
      */
     public GraduateLevel getGraduateLevel(String level){
        GraduateLevel status = null;
        if(level.equals("masters")){
            status = GraduateLevel.MASTERS;
        } else if(level.equals("phd")){
            status = GraduateLevel.PHD;
        } else {
            System.out.println("Invalid graduate level");
        }
        return status;
    }  
     
     /**
      * getUndergraduateLevel returns a UndergraduateLevel object for the String.
      * @param level A String for the undergraduate level can contain the 
      * value "freshman", "sophmore", "junior" or "senior"
      * @return UndergraduateLevel The Undergraduate students grade level.
      */
     public UndergraduateLevel getUndergraduateLevel(String level){
        UndergraduateLevel status = null;
        if(level.equals("freshman")){
            status = UndergraduateLevel.FRESHMAN;
        } else if(level.equals("sophmore")){
            status = UndergraduateLevel.SOPHMORE;
        } else if(level.equals("junior")){
            status = UndergraduateLevel.JUNIOR;
        }else if (level.equals("senior")){
            status = UndergraduateLevel.SENIOR;
        }else {
            System.out.println("Invalid graduate level");
        }
        return status;
    }      
}
