/*
 * Person.java
 *
 * Created on October 10, 2004, 11:05 PM
 */

package university;
import java.io.Serializable;

/**
 * Abstract class that represents people in the university.
 * @author Nathan Balon
 * University Application
 * CIS 550 - Object Oriented Programming
 * U 0f M Dearborn
 * November 7, 2004
 * @version 1.0
 */


public class Person implements Cloneable, Serializable{
    
    /** Creates a new instance of Person */
    public Person() {
        this.name = "NA";        
    }
    
    /** Creates a new instance of Person
     * @param name The name of the person
     * @param gender The sex of the person
     * @param address The address of the person
     * @param unit The unit the person belongs to
     */
    public Person(String name, Sex gender, Address address, Unit unit){
        id = ID.getFactory().getNextId();
        this.name = name;
        this.gender = gender;
        this.unit = unit;
        this.address = address;
    }
    
    /**
     * Clone a new object
     * @throws java.lang.CloneNotSupportedException Throws CloneNotSupportedException
     * @return Object a cloned object
     */    
    @Override
    public Object clone() throws java.lang.CloneNotSupportedException{
        Person cloned = (Person) super.clone();
        cloned.name = new String(name);
        cloned.unit = unit;
        cloned.address = (Address)address.clone();
        cloned.gender = gender;
        cloned.id = id;
        return cloned;
        
    }
 
    /**
     * Returns the name of a person
     * @return The name of the person
     */
    public final String getName(){
        return name;
    }
   
    /**
     * Returns the sex of a person
     * @return The gender of the person
     */
    public final Sex getGender(){
        return gender;
    }
    
    /**
     * Returns the unit a person belongs to.
     * @return The unit that the person belongs to
     */
    public final Unit getAffiliation(){
        return unit;
    }
    
    /**
     * Returns the ID of a person.
     * @return The ID of the person
     */
    public final int getID(){
        return id;
    }
    
    /**
     * Get the Address of the Person.
     * @return The Address of the Person.
     */
    public final Address getAddress(){
        return address;
    }
    
    /**
     * Sets the persons name to a new name.
     * @param name The new name for the person.
     */
    public final void setName(String name){
        this.name = name;
    }
    
    /**
     * Returns the String representation of a person.
     * @return A string representing the person
     */
    @Override
    public String toString(){
//        String per = name + " " + gender + " " + unit;
        return("Name: " + name + ", ID: " + id + 
               ", Gender: " + gender.toString().toLowerCase() + ", " + 
                address + "\nUnit: " + unit + "\n");
    }
    
    private String name;          // name of the person
    private Sex gender;           // gender of the person
    private Unit unit;            // unit the person belongs to
    private Address address;
    private int id;               // id of the person
    
}
