/*
 * RMIPersonClient.java
 *
 * Created on November 10, 2004, 2:14 PM
 */

package university;

import java.rmi.*;
import java.io.*;
import java.util.regex.*;
import java.util.ArrayList;

/**
 * PersonListClient is the client application for the RMI program.  The client
 * program creates new people for the university and using the PersonAdapter stores
 * the people that were created in the remote list PersonListImpl.
 *
 * @author  Nathan Balon
 * University Application
 * CIS 550 - Object Oriented Programming
 * U 0f M Dearborn
 * November 7, 2004
 * @version 1.0
 */

public class PersonListClient {
    
    /** Creates a new instance of RMIPersonClient */
    public PersonListClient() {
    }
    
    /** 
     * The main entry point to the client application of the remote list.
     * @param args The command line arguments.
     */
    public static void main(String [] args){
        PersonListClient client = new PersonListClient();
        ArrayList <PersonAdapter> localPersonList = new ArrayList<PersonAdapter>();
        // name of the registered object
        String name = "rmi://localhost/PersonList";
        // list of remote objects
        PersonList list = null;
        // create a new input stream to read user commands
        BufferedReader console = new BufferedReader(new InputStreamReader(System.in));
        System.out.println("University Remote List Application");
        System.out.println("Looking up " + name);
        try{
            // look up the remote list in the registry
            list = (PersonList)Naming.lookup(name);
        }catch(Exception e){
            // if the list is not found exit the program
            System.err.println("Exception looking up person list");
            e.printStackTrace();
            System.exit(1);
        }
        // create the units
        UnitsFactory.create();
        while(true){
            int choice = 0;                 // menu choice
            boolean isNumber = false;       // is user input valid
            while(!isNumber){
                String input = null;
                // dispaly the program menu to the user
                client.displayMenu();
                // get the users selection
                try{
                    input = console.readLine();
                }
                catch(IOException e){
                    System.err.println("Error reading from console");
                    e.printStackTrace();
                }
                // check that a valid entry was entered before parsing
                Pattern pattern = Pattern.compile("[1-4]");
                Matcher m = pattern.matcher(input);
                isNumber = m.matches();
                // check if a number was entered
                if(isNumber){
                    choice = Integer.parseInt(input);
                }else{
                    System.out.println("Invalid input");
                }   
            }
            switch(choice){
                case 1:
                    try{
                        // create a new person
                        Person p = (Person)PersonFactory.createPerson();
                        // use the adapter so the person can be used remotely
                        PersonAdapter person = new PersonAdapterImpl(p);
                        // add the person to the remote directory and local directory
                        localPersonList.add(person);
                        list.addPerson(person);
                        System.out.println("Added person to remote list");
                    }
                    catch(Exception e){
                        System.err.println("Error adding person to the list");
                        e.printStackTrace();
                    }
                    break;
                case 2:
                    try{
                        // list all people from the remote directory
                        System.out.println("\t\tList of all people in the remote list");
                        System.out.println("-------------------------------------" +
                                           "-------------------------------------");
                        System.out.println(list.ListAll());
                    }
                    catch(Exception e){
                        System.err.println("Error listing people in remote list");
                        e.printStackTrace();
                    }
                    break;
                case 3:
                    // change the name of the person to show that references will
                    // be changed on both the server and client since remote
                    // references are used as opposed to serialization
                    System.out.print("Enter the name of the person to change: ");
                    try{
                        // get the name of the person who name is wished to change
                        String nameOfPersonToChange = console.readLine();
                        // search for the person in the list
                        boolean personFound = false;
                        for(int i = 0; i < localPersonList.size(); i++){
                            PersonAdapter personToChange = localPersonList.get(i);
                            // if name match change the names
                            if(personToChange.getName().equals(nameOfPersonToChange)){
                                personFound = true;
                                System.out.println("Person found in list");
                                System.out.print("Enter the new name: ");
                                String newName = console.readLine();
                                personToChange.setName(newName);
                                System.out.println("\nNew updated information on the client:");
                                // display updated information
                                System.out.println(personToChange.printDescription());
                                System.out.println("To see that the change has also " +
                                            "taken place on the server, select list all\n");
                            }
                        }
                        if(!personFound){
                            System.out.println("The name of the person was not found in the list\n");
                        }
                    }
                    catch (Exception e){
                        e.printStackTrace();
                    }
                    break;
                case 4:
                    // exit the program
                    System.out.println("Closing the application");
                    System.out.println("Program clean up removing local references from the server list");
                    // remove all remote references to people created held locally
                    // from the server.
                    for(int i =0; i < localPersonList.size(); i++){
                        // get each person held locally
                        PersonAdapter personToRemove = localPersonList.get(i);
                        try{
                            // get the person id to be removed
                            int id = personToRemove.getID();
                            // remove the person from the server list
                            list.removePerson(id);
                            System.out.println("Removed " + personToRemove.getName());
                        }
                        catch(RemoteException e){
                            System.err.println("Error removing person from the list");
                        }
                    }
                    // exit the program
                    System.exit(0);
                default:
                    // should never get here
                    System.out.println("Invalid entry");
                    break;
            }   
        }
    }
    
    /** Menu display valid selections for user input */
    public void displayMenu(){
        System.out.println("\tRMI Application Menu");
        System.out.println("-------------------------------------");
        System.out.println("1 - add person");
        System.out.println("2 - display people");
        System.out.println("3 - change a persons name");   
        System.out.println("4 - exit");
        System.out.print("$ ");
    }
}
