/*
 * ProcessXML.java
 *
 * Created on December 12, 2004, 10:16 PM
 */

package university;

import javax.xml.*;
import org.w3c.dom.*;
import java.util.HashMap;

/**
 * ProcessDOM processes the nodes that are contained in the XML document.
 * The class can use DOM to process node for a Person, an Undergraduate
 * student and a Graduate student.
 * @author  Nathan Balon
 * University of Michigan Dearborn
 * CIS 550 Object Oriented Programming
 * 
 */

public class ProcessDOM {
    
    /** Creates a new instance of ProcessXML */
    public ProcessDOM() {
    }
    /**
     * processUndergraduate process a undergraduate Node of the XML document by
     * reading all the information for the student and the return a HashMap
     * of all the values read.
     * @param undergraduateNode The undergraduate Node from the XML document.
     * @return HashMap A HashMap of all the values read in from the XML node
     * for an undergraduate student.
     */
    public HashMap <String, String> processUndergraduate(Node undergraduateNode){
        String status = null;
        String name = null;
        String gpa = null;
        if(undergraduateNode.getNodeName().equals("undergraduate")){
             NamedNodeMap map = undergraduateNode.getAttributes();
             Node nameNode = map.getNamedItem("name");
             name = nameNode.getTextContent();
             nameNode = map.getNamedItem("gpa");
             gpa = nameNode.getTextContent();
        }
        NodeList undergraduateNodes = undergraduateNode.getChildNodes();
        for(int i = 0; i < undergraduateNodes.getLength(); i ++){
            Node node = undergraduateNodes.item(i); 
            if(node.getNodeType() == node.ELEMENT_NODE){
                if(node.getNodeName().equals("status")){
                    status = node.getTextContent();
                } 
            }
        }
        HashMap <String, String> undergraduateMap = this.processSharedElements(undergraduateNode);
        undergraduateMap.put("name", name);
        undergraduateMap.put("gpa", gpa);
        undergraduateMap.put("status", status);
        return undergraduateMap;
    }       
    
    
    /**
     * processGraduate processes a graduate Node of the XML document
     * reading all the information for the student and returning HashMap of
     * the values read.
     * @param graduateNode The graduate Node from the XML document.
     * @return HashMap A HashMap of all the values read in from the XML node
     * for an undergraduate student.     
     */
    public HashMap <String, String> processGraduate(Node graduateNode){
        String status = null;
        String name = null;
        String research = null;
        if(graduateNode.getNodeName().equals("graduate")){
             NamedNodeMap map = graduateNode.getAttributes();
             Node nameNode = map.getNamedItem("name");
             name = nameNode.getTextContent();
        }
        NodeList graduateNodes = graduateNode.getChildNodes();
        for(int i = 0; i < graduateNodes.getLength(); i ++){
            Node node = graduateNodes.item(i); 
            if(node.getNodeType() == node.ELEMENT_NODE){
                if(node.getNodeName().equals("status")){
                    status = node.getTextContent();
                } else if(node.getNodeName().equals("researchTopic")){
                    research = node.getTextContent();
                }
            }
        }
        HashMap <String, String> graduateMap = this.processSharedElements(graduateNode);
        graduateMap.put("name", name);
        graduateMap.put("status", status);
        graduateMap.put("researchTopic", research);
        return graduateMap;
    }
    
    /**
     * processPerson processes a person Node of the xml document by reading
     * in all the values of the person and inserting the values into a 
     * HashMap and then return the HashMp.
     * @param personNode The personNode of the xml document.
     * @return HashMap A HashMap of all the values read in from the XML node
     * for a person.
     */
    public HashMap <String, String> processPerson(Node personNode){
        String name = null;
        if(personNode.getNodeName().equals("person")){
             NamedNodeMap map = personNode.getAttributes();
             Node nameNode = map.getNamedItem("name");
             name = nameNode.getTextContent();
        }
        HashMap <String, String> personMap = this.processSharedElements(personNode);
        personMap.put("name", name);
    //    System.out.println(personMap);
        return personMap;
    }
    
    /**
     * processSharedElements process the personNode by processing all
     * the elemements that are shared by all objects
     * @param personNode A node contianing all of the elements that are
     * shared by all of the object derived from person.
     * @return HashMap A HashMap contain all of the values read in from the
     * xml node.
     */
    public HashMap <String, String> processSharedElements(Node personNode){
        String street = null;
        String city = null;
        String state = null;
        String zipCode = null;
        String unit = null;
        String gender = null;
        NodeList personNodes = personNode.getChildNodes();
        for(int i = 0; i < personNodes.getLength(); i ++){
            Node node = personNodes.item(i);
            if(node.getNodeType() == Node.ELEMENT_NODE){
                if(node.getNodeName().equals("address")){
                    NodeList addressNodes = node.getChildNodes();
                    for(int j = 0; j < addressNodes.getLength(); j++){
                        Node addressNode = addressNodes.item(j);
                        if(addressNode.getNodeName().equals("street")){
                            street = addressNode.getTextContent();                            
                        } else if(addressNode.getNodeName().equals("city")){
                            city = addressNode.getTextContent();                            
                        } else if(addressNode.getNodeName().equals("state")){
                            state = addressNode.getTextContent();                            
                        } else if(addressNode.getNodeName().equals("zipcode")){
                            zipCode = addressNode.getTextContent();
                        } 
                    }                    
                } else if(node.getNodeName().equals("gender")){
                    gender = node.getTextContent();
                } else if(node.getNodeName().equals("unit")){
                    unit = node.getTextContent();
                }
            }
        }
        HashMap <String, String> elementMap = new HashMap<String, String>();
        elementMap.put("street", street);
        elementMap.put("city", city);
        elementMap.put("state", state);
        elementMap.put("zipcode", zipCode);
        elementMap.put("unit", unit);
        elementMap.put("gender", gender);
        return elementMap;
    }    
}
