/*
 * Student.java
 *
 * Created on October 10, 2004, 11:07 PM
 */

package university;

/**
 * @author  Nathan Balon
 * University Application
 * CIS 550 - Object Oriented Programming
 * U 0f M Dearborn
 * November 7, 2004
 * @version 1.0
 */

/**
 * Abstract class for the students in the university.
 */
abstract public class Student extends Person implements Cloneable {
    
    /** Creates a new instance of Student */
    public Student() { 
        super();
        this.canRegister = true;        
    }
    
    /**
     * Creates a new instance of Student.
     * @param aName The name of the Student.
     * @param aGender The sex of the Student.
     * @param address The address of the Student.
     * @param aUnit The unit the Student belongs to.
     */
    public Student(String aName, Sex aGender, Address address, Unit aUnit){
        super(aName, aGender, address, aUnit);
        this.canRegister = true;
    }
    
    /**
     * Clone a new object
     * @throws java.lang.CloneNotSupportedException Throws CloneNotSupportedException
     * @return Object a cloned object
     */    
    @Override
    public Object clone() throws java.lang.CloneNotSupportedException{
        Student cloned = (Student)super.clone();
        cloned.canRegister = canRegister;
        return cloned;
    }
    
    /**
     * Set if the Student is allowed to register for class.
     * @param canRegister Set if the Student can register for class. 
     */
    public final void setAllowedTORegigister(boolean canRegister){        
        this.canRegister = canRegister;
    }
    
    /**
     * Checks if the Student is allowed to register for classes.
     * @return boolen Is the Student allowed to register.
     */
    public final boolean isAllowedToRegister(){
        return canRegister;
    }
    
    /**
     * Returns A String representation of a Student.
     * @return String A String representing a Student.
     */
    @Override
    public String toString(){
        String registrationStatus;
        if(canRegister){
            registrationStatus = "Allowed to register";
        }
        else{
            registrationStatus = "Hold on registation";
        }
        return(super.toString() + "Registration Status: " + registrationStatus);
    }
    
    boolean canRegister;            // is the student allowed to register
}
