/*
 * UniversityXMLParser.java
 *
 * Created on November 26, 2004, 6:15 AM
 */

package university;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.validation.*;
import javax.xml.XMLConstants;
import javax.xml.*;
import org.w3c.dom.*;
import java.util.HashMap;
import java.util.ArrayList;
import java.io.File;
import org.xml.sax.*;

/**
 * UniversityXMLParser reads in an XML document containing members of the
 * University.  The XML file is validated against a XML schema to determine
 * if the document is well formed and correct.  The program then creates a
 * new person type of person depending on the type of node that is read.  For
 * example if a Undergraduate node is read the program will create a new
 * Undergraduate object.  When the program terminates a list of the people
 * created is displayed to the user.<br /><br />
 * 
 * To run the program:<br/><br/>
 * java university/UniversityXMLParser [xmlDocument] [xmlSchema]<br />
 *
 * @author  nathan Balon
 * University of Michigan Dearborn<br />
 * CIS 550 Object Oriented Programming<br />
 */

public class UniversityXMLParser {
    
    /** Creates a new instance of UniversityXMLParser */
    public UniversityXMLParser() {
        // create all of the units of the university
        UnitsFactory.create();
    }
        
    /**
     * Display all of the people that were created from the XML document.
     * @param universityList A list of all of the people that were created.
     */
    public void display(ArrayList <Person> universityList){
        System.out.println("\n\t\tUniversity members created from XML file");
        System.out.println("-----------------------------------------------" +
                           "-----------------------------------------------");
        for(int i = 0; i < universityList.size(); i++){
            System.out.println(universityList.get(i));
        }
    }
    
     /**
      * Entry point to the UniversityXMLParser application.
      * To run the program: university/UniversityXMLParser [xmlDocument] [xmlSchema]
      * @param args the command line arguments
      */
    public static void main(String[] args) throws Exception{
        // a list to hold all of the people created
        ArrayList <Person> universityList = new ArrayList<Person>();
        UniversityXMLParser universityXML = new UniversityXMLParser();
        // check that the xml document and schema was supplied
        if(args.length != 2){
            System.err.println("Usage: UniversityXMLParser [XML file] [XML Schema]");
            System.exit(0);
        }
        // get the path for the xml document and schema
        String xmlFilePath = args[0];
        String xmlSchemaPath = args[1];
        SchemaFactory sf = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
        // set the error handlers for the schema validation
	sf.setErrorHandler(new org.xml.sax.ErrorHandler() {
            public void warning(SAXParseException e) {
                System.err.println("WARNING: " + e.getMessage());
            }
            public void error(SAXParseException e) {
                System.err.println("ERROR: " + e.getMessage());
                System.exit(1);
            }
            public void fatalError(SAXParseException e)
                throws SAXException {
                System.err.println("FATAL: " + e.getMessage());
                throw e;   // re-throw the error
            }
        });
	Schema schema = sf.newSchema(new File(xmlSchemaPath));
	DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	factory.setNamespaceAware(true);
	factory.setSchema(schema);
	DocumentBuilder parser = factory.newDocumentBuilder();
        // set the error handlers for the parser
        parser.setErrorHandler(new org.xml.sax.ErrorHandler() {
            public void warning(SAXParseException e) {
                System.err.println("WARNING: " + e.getMessage());
            }
            public void error(SAXParseException e) {
                System.err.println("ERROR: " + e.getMessage());
                System.err.println("The XML document is not valid");
                System.exit(1);
            }
            public void fatalError(SAXParseException e)
                throws SAXException {
                System.err.println("FATAL: " + e.getMessage());
                throw e;   // re-throw the error
            }
        });
	Document xmlDocument = parser.parse(xmlFilePath);
        try{ 
            // create the object to process the nodes of the document
            ProcessDOM processDOM = new ProcessDOM();
            // create an object to create a Person object from
            // the xml document.
            BuildPersonFromXML buildFromXML = new BuildPersonFromXML();            
            // get the root element for the xml document
            Node rootElement = xmlDocument.getDocumentElement();
            // get all the child nodes of the root
            NodeList personList = rootElement.getChildNodes();
            for(int i = 0; i < personList.getLength(); i++){
                // get a node from the person list xml document
                Node personListNode = personList.item(i);
                // if the node is an element process the node
                if(personListNode.getNodeType() == Node.ELEMENT_NODE){
                    // get the name of the node
                    String name = personListNode.getNodeName();
                        // if the node is a person, parse the person node
                    if(name.equals("person")){
                        HashMap <String, String> personMap =
                                 processDOM.processPerson(personListNode);
                        Person p = buildFromXML.createPerson(personMap);
                        universityList.add(p);
                        // if the node is a graduate parse the graduate node
                    } else if(name.equals("graduate")){
                        HashMap <String, String> graduateMap =
                                processDOM.processGraduate(personListNode);
                        Graduate g = buildFromXML.createGraduate(graduateMap);
                        universityList.add(g);
                        // if the node is an undergraduate parse the undergraduate node
                    } else if(name.equals("undergraduate")){
                        HashMap <String, String> undergraduateMap =
                              processDOM.processUndergraduate(personListNode);
                        Undergraduate u = buildFromXML.createUndergraduate(undergraduateMap);
                        universityList.add(u);
                    }       
                }                
            }
            // display all of the people that were created
            universityXML.display(universityList);            
        }   
        catch(Exception e){
            System.err.println("Exception parsing the XML file");
            e.printStackTrace();
        }
        
    }
}
